Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

entity Circular_Buffer_16bit is 
	port( 	
			clk				: in std_logic;
			nreset			: in std_logic;
			
			Data_In			: in std_logic_vector(15 downto 0);
			wr				: in std_logic;
			rd				: in std_logic;
			reset_buffer	: in std_logic;

			Data_Out		: out std_logic_vector(15 downto 0);
			Empty_Flag 		: out std_logic;
			Full_Flag  		: out std_logic;
			Buf_Cnt    		: out std_logic_vector(10 downto 0);
			In_Ptr     		: out std_logic_vector(10 downto 0);
			Out_Ptr    		: out std_logic_vector(10 downto 0)
			        
		);
	end Circular_Buffer_16bit;
	
architecture main of Circular_Buffer_16bit is	

	signal data_ptr_in	: std_logic_vector(10 downto 0); -- address 4096 memory locations
	signal data_ptr_out : std_logic_vector(10 downto 0);
	
	signal inc_ptr_in	: std_logic;
	signal inc_ptr_out	: std_logic;
	signal inc_ptr_out2	: std_logic;
	
  	signal wren			: std_logic;
  	signal rden     	: std_logic;
  	signal init_rd  	: std_logic;
  	signal init_rd2 	: std_logic;

	signal buffer_cnt	: std_logic_vector(10 downto 0);
	signal inc_buf_cnt 	: std_logic;
	signal dec_buf_cnt	: std_logic;
	
	signal empty 		    : std_logic;
	signal data_delay  : std_logic;
	signal full  		    : std_logic;

	component ram_2port_16x2048
		PORT
		(
			data		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
			rdaddress	: IN STD_LOGIC_VECTOR (10 DOWNTO 0);
			rdclock		: IN STD_LOGIC ;
			rdclocken	: IN STD_LOGIC  := '1';
			wraddress	: IN STD_LOGIC_VECTOR (10 DOWNTO 0);
			wrclock		: IN STD_LOGIC ;
			wrclocken	: IN STD_LOGIC  := '1';
			wren		: IN STD_LOGIC  := '1';
			q			: OUT STD_LOGIC_VECTOR (15 DOWNTO 0)
		);
	end component;
	
	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, inc_in_ptr, update_rd2);
	signal CS, NS  : STATE_TYPE;

	type STATE_TYPE2 is 
	(init2, idle2, inc_out_ptr, pause);
	signal CS2, NS2  : STATE_TYPE2;

--==========================================================================
--              B E G I N 
--==========================================================================          
begin

CURVE_MEM: ram_2port_16x2048
port map(
		 data => 		Data_In,
		 rdaddress => 	data_ptr_out, 
		 rdclock =>		clk,
		 rdclocken =>	rden,
		 wraddress => 	data_ptr_in, 
		 wrclock =>		clk,
		 wrclocken =>	wren,
		 wren => 		'1',
		 q => 			Data_Out
		);
wren <= wr AND NOT full;
rden <= rd OR init_rd OR init_rd2;

-- Data Pointer Control ----------------------------
BUFFER_COUNT:
process(clk, nreset, buffer_cnt, inc_buf_cnt, dec_buf_cnt, reset_buffer)
begin
	if   (nreset <= '0') OR (reset_buffer = '1') then
		buffer_cnt  <= (others => '0');
	elsif rising_edge(clk) then
		if   (inc_buf_cnt = '1') AND (dec_buf_cnt = '0') then	
			buffer_cnt <= buffer_cnt + 1;
		elsif(inc_buf_cnt = '0') AND (dec_buf_cnt = '1') then	
			buffer_cnt <= buffer_cnt - 1;
		else
			buffer_cnt <= buffer_cnt;
		end if;
	end if;
	---------------------
	if   (buffer_cnt = "11111111111") then 
		full 	<= '1';
		empty 	<= '0';
	elsif(buffer_cnt = "00000000000") then
		full 	<= '0';
		empty 	<= '1';
	else
		full 	<= '0';
		empty 	<= '0';
	end if;
end process;

---------------------------------------------------
DATA_POINTER_IN:
process(clk, nreset, inc_ptr_in, reset_buffer)
begin
	if   (nreset <= '0') OR (reset_buffer = '1') then
		data_ptr_in  <= (others => '0');
	elsif rising_edge(clk) then
		if(inc_ptr_in = '1') then 	data_ptr_in <= data_ptr_in  + 1;
		else						data_ptr_in <= data_ptr_in;
		end if;
	end if;
end process;
---------------------------------------------------
DATA_POINTER_OUT:
process(clk, nreset, inc_ptr_out, reset_buffer)
begin
	if   (nreset <= '0') OR (reset_buffer = '1') then
		data_ptr_out  <= (others => '0');
	elsif rising_edge(clk) then
		if(inc_ptr_out = '1') OR (inc_ptr_out2 = '1') then 	
									data_ptr_out <= data_ptr_out  + 1;
		else						data_ptr_out <= data_ptr_out;
		end if;
	end if;
end process;

-- State Machine Process definition ---------------------------
	sm_reg: process (clk, nreset)	
	begin
		if(nreset='0')              then CS <= idle;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: process (CS, wr, full, empty)
	begin
		inc_ptr_in  	<= '0';
		inc_ptr_out2	<= '0';
		inc_buf_cnt		<= '0';
		init_rd2     <= '0';
		
		case CS is
			when idle   =>
				if(wr = '1') then 		NS <= inc_in_ptr;
				else					NS <= idle;
				end if;
				 
			-----	
			when inc_in_ptr =>			
				inc_ptr_in <= '1';
				
				if   (full = '1') then  
				    inc_ptr_out2 <= '1'; 	NS <= update_rd2;
				    
				elsif(empty = '1') then
				    inc_buf_cnt  <= '1';	NS <= update_rd2;
				       
				else					
				    inc_buf_cnt  <= '1';  	NS <= idle;
				end if;
				
			-----
			when update_rd2 =>          	NS <= idle;
				init_rd2     <= '1';
				
			-----	
			when others =>					NS <= idle;
		end case;
	end process;
----------------------------------------------------
-- State Machine Process definition ---------------------------
	sm_reg2: process (clk, nreset)	
	begin
		if(nreset='0')              then CS2 <= init2;
		elsif(clk'event and clk='1')then CS2 <= NS2;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial2: process (CS2, rd, empty)
	begin
		inc_ptr_out  	<= '0';
		dec_buf_cnt		<= '0';
		init_rd     <= '0';
		
		case CS2 is
			when init2   =>				NS2 <= idle2;
			  init_rd  <= '1';
				
			-----	
			when idle2   =>
				if(rd = '1') then 		NS2 <= inc_out_ptr;
				else					NS2 <= idle2;
				end if; 
				
			-----	
			when inc_out_ptr =>			NS2 <= pause;
				if   (empty = '0') then 
					inc_ptr_out	<= '1'; 
					dec_buf_cnt <= '1';
				end if;
				
			-----	
			when pause =>
				init_rd <= '1';
				if(rd = '0') then		NS2 <= idle2;
				else					NS2 <= pause;
				end if;
				
			-----	
			when others =>				NS2 <= idle2;
		end case;
	end process;
---------------------------------------------------
HOLD_FOR_DATA_READY:
process(clk, nreset, empty)
begin
	if   (nreset <= '0') then data_delay  <= '0';
	elsif rising_edge(clk) then
      data_delay <= empty;
	end if;    
end process;

	  Empty_Flag <= empty OR data_delay;
      Full_Flag  <= full;
      Buf_Cnt    <= buffer_cnt;
      In_Ptr     <= data_ptr_in;
      Out_Ptr    <= data_ptr_out;

end;