--------------------------------------------------------------------
--  This routine will generate a curve from ROM memory.  The ROM
--  memory is written during compile time from the following file:
--
--         "curve_mem2.mif" <-- 256, 11 bit words
--
--  A counter is used to sequence throught the words in memory.
--  The counter is a pointer into memory.  The address update rate
--  can be set from 0 to 65,535 system clocks, "intv_clks".
--  The sequence begins with the "Start_Trig" signal and is reset
--  by the "End_Trig".  The sequence holds when the address reaches 254.

--  NOTE NOTE NOTE :  
--  The gain values are loaded into the AD5415 Multiplying DAC's run
--  in the Bipolar Mode.  The input is to be coded as an Offset Binary
--  number.

--   Digital Input   |  Analog Output
--   ------------------------------------
--   1111 1111 1111  |  Vref*(2047/2048)
--   1111 1111 1110  |  Vref*(2046/2048)
--         ...       |        ...
--   1000 0000 0000  |        0
--   0111 1111 1111  |  Vref*( -1 /2048)
--         ...       |        ...
--   0000 0000 0001  |  Vref*(-2047/2048)
--   0000 0000 0000  |  Vref*(-2048/2048)

--   ** In using the Altera signed multiply functions these values need
--      to be converted to 2's Complement.  This is done simply by
--      inverting the most significant bit (bit 11).

-- -----------------------------------------------------------------
-- Revision 10/29/2010
-- Additional curve memory blocks were added to provide more options
-- for transfering between acceleration phase error and the MI phase
-- error.  The additional curve file is "curve_mem1_alt.mif".  A control
-- bit was added to the inputs to the block to choose this curve or
-- the original one.

Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

entity CurveAGen40 is
	port(clk        	: in std_logic;
		 nreset     	: in std_logic;
		 Start_Trig 	: in std_logic;
		 End_Trig   	: in std_logic;
		 intv_clks  	: in std_logic_vector(15 downto 0);
		 base_gain  	: in std_logic_vector(11 downto 0);
		 NO_BEAM_OFFS   : in std_logic_vector(13 downto 0);
		 NO_BEAM		: in std_logic;
		 Use_Alt_Curve	: in std_logic;
		 Force_Active   : in std_logic;
		 Force_Zero		: in std_logic;
		 Beam_Cyc_Reset : in std_logic;
		 
		 MDAC_Curve_Out : out std_logic_vector(15 downto 0);
		 NO_BEAM_OFFSET : out std_logic_vector(15 downto 0);
		 Enable_chan	: out std_logic
		 --result_2s  : out std_logic_vector(23 downto 0);
		 --rom_out    : out std_logic_vector(15 downto 0);
		 --rom_out_2s : out std_logic_vector(11 downto 0);
		 --base_2s    : out std_logic_vector(11 downto 0)
		);
end CurveAGen40;
	
architecture main of CurveAGen40 is

------------------------------------------------------------
-- ALTERA COMPONENTS ---------------------------------------
-- The following components are the ROM memory and a multiplier
-- They were created using the LPM Megafunction Wizard.
-- The following VHDL files were generated and must be included 
-- in the project.
--                              curve_mem1.vhd
--                            mult12signed.vhd
--                            mult14signed.vhd
--
-- The Megafunction Wizard also generate *.cmp files for each
-- from which I cut and pasted the component declarations below.
----------------------------------------------------------------
	component curve_mem1
		PORT
		(
			address		: IN STD_LOGIC_VECTOR (7 DOWNTO 0);
			clock		: IN STD_LOGIC ;
			data		: IN STD_LOGIC_VECTOR (11 DOWNTO 0);
			wren		: IN STD_LOGIC ;
			q		: OUT STD_LOGIC_VECTOR (11 DOWNTO 0)
		);
	end component;

	component curve_mem1_alt
		PORT
		(
			address		: IN STD_LOGIC_VECTOR (7 DOWNTO 0);
			clock		: IN STD_LOGIC ;
			data		: IN STD_LOGIC_VECTOR (11 DOWNTO 0);
			wren		: IN STD_LOGIC ;
			q		: OUT STD_LOGIC_VECTOR (11 DOWNTO 0)
		);
	end component;

	component mult12signed
		PORT
		(
			clock		: IN STD_LOGIC ;
			dataa		: IN STD_LOGIC_VECTOR (11 DOWNTO 0);
			datab		: IN STD_LOGIC_VECTOR (11 DOWNTO 0);
			result		: OUT STD_LOGIC_VECTOR (23 DOWNTO 0)
		);
	end component;

	component mult14signed
	PORT
	(
		dataa		: IN STD_LOGIC_VECTOR (13 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (13 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (27 DOWNTO 0)
	);
	end component;

-----------------------------------------------------------------

	-- ADDR_CNT Counter/Timer declarations ---------------------------
	-- The output of this counter is the address pointer
	signal nclr_adr : std_logic;  -- output of sm
	signal inc_adr  : std_logic;  -- output of sm
	signal end_flag : std_logic;  -- output of ADDR_CNT, input to sm
	signal last_adr : std_logic_vector(7 downto 0);  -- input to ADDR_CNT
	signal addr     : std_logic_vector(7 downto 0);  -- output of ADDR_CNT
	
	-- INTERVAL_CNT declarations ------------------------------
	signal nclr_cnt : std_logic; -- output of sm
	signal smp_flag : std_logic; -- output of INTERVAL_CNT, input to sm

	-- Sync signal declarations --------------------------------------
	signal go        : std_logic;  -- input to sm
	signal end_cycle : std_logic; -- input to sm

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(init, idle, wait_1, nextadr, hold, rst_adr);
	signal CS, NS  : STATE_TYPE;
	
	-- ROM Curve intermediate signals -------------------------------
	signal curve   		: std_logic_vector(11 downto 0);
	signal curve_0		: std_logic_vector(11 downto 0);
	signal curve_1   	: std_logic_vector(11 downto 0);
	
	-- Multiplier intermediate signals ------------------------------
	signal mode			: std_logic_vector(2 downto 0);
	
	signal base_gain_2s : std_logic_vector(11 downto 0);
	signal curve_2s     : std_logic_vector(11 downto 0);
	signal product_2s   : std_logic_vector(23 downto 0);


	signal dac_offset_2s	: std_logic_vector(13 downto 0);
	signal dac_product_2s  	: std_logic_vector(27 downto 0);

-- ## MAIN #########################################################
begin 
	-- convert 12 bit offset binary to 12 bit 2's comlement
	base_gain_2s <= not base_gain(11) & base_gain(10 downto 0);
	
	-- convert 14 bit offset binary to 12 bit 2's comlement
	dac_offset_2s	<= not NO_BEAM_OFFS(13) & NO_BEAM_OFFS(12 downto 0);

------------------------------------------------------------
-- ALTERA COMPONENTS ---------------------------------------
	CURVE_MEM: curve_mem1
	port map(address => addr, 
			 clock => clk, 
			 data => (others => '0'),
			 wren => '0',
			 q => curve_0(11 downto 0)
			);
		
	CURVE_MEM_ALT: curve_mem1_alt
	port map(address => addr, 
			 clock => clk, 
			 data => (others => '0'),
			 wren => '0',
			 q => curve_1(11 downto 0)
			);

	G1xCURVE : mult12signed 
	port map (
				clock	 => clk,
				dataa	 => base_gain_2s,
				datab	 => curve_2s,
				result	 => product_2s
			 );
			
	----------------------------------------		
	G1xINT_CURVE : mult14signed 
	port map (
				dataa	 => dac_offset_2s, --<= not NO_BEAM_OFFS(13) & NO_BEAM_OFFS(12 downto 0);
				datab	 => curve_2s & "00",
				result	 => dac_product_2s
			 );
			
		
	process(Force_Active, Force_Zero, Use_Alt_Curve)
	begin
		if   (Force_Active = '1')  then mode <= "100";			
		elsif(Force_Zero = '1')    then mode <= "010";
		elsif(Use_Alt_Curve = '1') then mode <= "001";	
		else							mode <= "000";					
		end if;
	end process;
	
	process(mode, curve_0, curve_1)
	begin
		CASE mode IS
			when "000" => curve_2s <= (not curve_1(11) & curve_1(10 downto 0));
			when "001" => curve_2s <= (not curve_0(11) & curve_0(10 downto 0));
			when "010" => curve_2s <= X"000";
			when "100" => curve_2s <= X"7FF";
			when others => curve_2s <= (not curve_1(11) & curve_1(10 downto 0));
		end case;
	end process;

--	process(product_2s)
--	begin
--		if(product_2s = X"000") then 	Enable_chan <= '0';
--		else							Enable_chan <= '1';
--		end if;
--	end process;
	Enable_chan <= '1';
-------------------------------------------------------------	
		
	ADDR_CNT: counter  -- address sequencer
	-- The output of this counter is the address pointer
	generic map(WID8)
	port map ( clk=>clk, clrn=>nclr_adr, ena=>inc_adr, load=>'0', updn=>'1',
			   ld_data  => (others => '0'),
			   flag_cnt => last_adr,
			   flag_out => end_flag,
			   cnt_out  => addr
			 );
	last_adr <= X"FD";
	
	INTERVAL_CNT: counter  -- Defines the curve sample interval
	generic map(WID16)
	port map ( clk=>clk, clrn=>nclr_cnt, ena=>'1', load=>'0', updn=>'1',
			   ld_data  => (others => '0'),
			   flag_cnt => intv_clks,
			   flag_out => smp_flag,
			   cnt_out  => open
			 );

	TRIGGER_SYNC: led  -- Syncronizes the start trigger input
	port map(sig_in => Start_Trig, clk => clk, clrn => '1', pls_out => go );
	
	END_CYCLE_SYNC: led  -- Syncronizes the end cycle trigger input
	port map(sig_in => End_Trig, clk => clk, clrn => '1', pls_out => end_cycle );
		
-- State Machine Process definition ---------------------------
	sm_reg: process (clk, nreset)	
	begin
		if(nreset='0')              then CS <= init;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: process (CS, go, smp_flag, end_flag, end_cycle)
	begin
		case CS is
			when init   =>
				NS <= idle  ;
				-- outputs --
				nclr_cnt <= '0'; -- counter is reset
				nclr_adr <= '0'; -- counter is reset
				inc_adr  <= '0'; -- not incrementing the address pointer
				--------------------
			when idle   =>
				if(go ='1')          then NS <= wait_1  ;
				else                      NS <= idle;
				end if; 
				-- outputs --
				nclr_cnt <= '0'; -- counter is reset
				nclr_adr <= '0'; -- counter is reset
				inc_adr <= '0';  -- not incrementing the address pointer
				--------------------
			when wait_1 => -- wait for next sample time
				if(smp_flag ='1')    then NS <= nextadr;
				elsif(end_cycle ='1')then NS <= rst_adr;
				else                      NS <= wait_1;
				end if;
				-- outputs --
				nclr_cnt <= '1'; -- not reset, but waiting
				nclr_adr <= '1';
				inc_adr <= '0';
				--------------------
			when nextadr => 
				if(end_flag ='1')    then NS <= hold;
				elsif(end_cycle ='1')then NS <= rst_adr;
				else                      NS <= wait_1;
				end if;
				-- outputs --
				nclr_cnt <= '0'; -- clear the interval count
				nclr_adr <= '1';
				inc_adr <= '1';  -- enable the addr count
				--------------------
			when hold => -- Hold last address until end of cycle trigger
				if(end_cycle ='1')   then NS <= rst_adr;
				else                      NS <= hold;
				end if;
				-- outputs --
				nclr_cnt <= '1';
				nclr_adr <= '1';
				inc_adr <= '0';
				--------------------
			when rst_adr => -- A two second press will reset the module
				NS <= idle;
				-- outputs --
				nclr_cnt <= '0'; -- clear all counters
				nclr_adr <= '0'; -- clear all counters
				inc_adr <= '0';
				--------------------
		end case; 
				
	end process; -- sm_combinatorial
	
-- Syncronized Outputs ----------------------------
process(clk, NO_BEAM, dac_product_2s, product_2s, Beam_Cyc_Reset)
begin
if   (NO_BEAM = '1') then
	MDAC_Curve_Out(15 downto 0) <= "0000" & X"800"; -- 12 bit offset binary zero
										
	if (Beam_Cyc_Reset = '1') then
	NO_BEAM_OFFSET(15 downto 0) <= "00" & not dac_product_2s(27) & dac_product_2s(25 downto 13);
											--^-- padded 14 bit offset binary normalized result
	end if;
											
elsif rising_edge(clk) then
	MDAC_Curve_Out(15 downto 0) <= "0000" & not product_2s(23) & product_2s(21 downto 11);
									 
									 
	NO_BEAM_OFFSET(15 downto 0) <=   X"2000"; -- 14 bit offset binary zero
											

									 
	--result_2s  <= product_2s;
	--rom_out    <= curve; --: out std_logic_vector(15 downto 0);
	--rom_out_2s <= curve_2s; -- : out std_logic_vector(11 downto 0);
	--base_2s    <= base_gain_2s; --: out std_logic_vector(11 downto 0)
	
end if;
end process;


end main;

