Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

entity Flash_Bus_Management is 
	port( 	
			clk					: in std_logic;
			nreset				: in std_logic;
			SaveAll				: in std_logic;
			RestoreAll			: in std_logic;
			Xfer_Done			: in std_logic;
			FLSH_Bus_Permit		: in std_logic;
			
			SaveAll_out			: out std_logic;
			RestoreAll_out		: out std_logic;
			FLSH_Bus_Request	: out std_logic
		);
	end Flash_Bus_Management;
	
architecture main of Flash_Bus_Management is	
	
	signal Bootup			: std_logic;
	signal sync_save		: std_logic_vector(2 downto 0);
	signal sync_restore		: std_logic_vector(2 downto 0);
	signal do_save			: std_logic;
	signal do_restore		: std_logic;
	signal ltch_save		: std_logic;
	signal ltch_restore		: std_logic;
	
	signal sync_done		: std_logic_vector(1 downto 0);
	signal done				: std_logic;
	
	signal sync_permit		: std_logic_vector(2 downto 0);
	signal bus_permit		: std_logic;
	
	signal delay			: std_logic_vector(15 downto 0);
	signal ena_delay		: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(init, idle, make_request, save_active, restore_active, transf_done);
	signal CS, NS  : STATE_TYPE;

--======================================================================
--               B  E  G  I  N  
------------------------------------------------------------------------       
begin

------------------------------------------------------
	SYNC_BUS_PERMIT_INPUT:
	process(clk, nreset, FLSH_Bus_Permit, sync_permit) 
	begin
		if (nreset = '0') then
			sync_permit <= (others => '0');
		elsif rising_edge(clk) then
			sync_permit(0) <= FLSH_Bus_Permit;
			sync_permit(1) <= sync_permit(0);
			sync_permit(2) <= sync_permit(1);
		end if;			
			bus_permit <= sync_permit(1) AND NOT sync_permit(2);
	end process;

------------------------------------------------------
	SYNC_SAVE_COMMAND_INPUT:
	process(clk, nreset, SaveAll, sync_save) 
	begin
		if (nreset = '0') then
			sync_save <= (others => '0');
		elsif rising_edge(clk) then
			sync_save(0) <= SaveAll;
			sync_save(1) <= sync_save(0);
			sync_save(2) <= sync_save(1);
		end if;			
			do_save <= sync_save(1) AND NOT sync_save(2);
	end process;

------------------------------------------------------
	SYNC_RESTORE_COMMAND_INPUT:
	process(clk, nreset, RestoreAll, sync_restore) 
	begin
		if (nreset = '0') then
			sync_restore <= (others => '0');
		elsif rising_edge(clk) then
			sync_restore(0) <= RestoreAll;
			sync_restore(1) <= sync_restore(0);
			sync_restore(2) <= sync_restore(1);
		end if;			
			do_restore <= sync_restore(1) AND NOT sync_restore(2);
	end process;

------------------------------------------------------
	SYNC_XFER_DONE_INPUT:
	process(clk, nreset, Xfer_Done, sync_done) 
	begin
		if (nreset = '0') then
			sync_done <= (others => '0');
		elsif rising_edge(clk) then
			sync_done(0) <= Xfer_Done;
			sync_done(1) <= sync_done(0);
		end if;			
			done <= sync_done(0) AND NOT sync_done(1);
	end process;

------------------------------------------------------
	LATCH_SAVE_COMMAND_INPUT:
	process(clk, nreset, do_save, done)
	begin
		if (nreset = '0') then 		  ltch_save <= '0';
		elsif rising_edge(clk) then
			if   (done    = '1') then ltch_save <= '0';			
			elsif(do_save = '1') then ltch_save <= '1';
			end if;
		end if;			
	end process;
	
------------------------------------------------------
	LATCH_RESTORE_COMMAND_INPUT:
	process(clk, nreset, do_restore, done)
	begin
		if (nreset = '0') then
			ltch_restore <= '0';
		elsif rising_edge(clk) then
			if   (done       = '1') then ltch_restore <= '0';			
			elsif(do_restore = '1') then ltch_restore <= '1';				
			elsif(Bootup     = '1') then ltch_restore <= '1';				
			end if;
		end if;			
	end process;

------------------------------------------------------------------
	STARTUP_RESET_DELAY:
	process(clk, nreset, ena_delay)
	begin
		if  (nreset = '0') then delay <= (others => '0');
		elsif rising_edge(clk) then
			if   (ena_delay = '1') then delay <= delay + 1;
			else						delay <= (others => '0');
			end if;
		end if;
	end process;
	
-- State Machine define initialization ---------------------------
	sm_reg: process (clk, nreset)	
	begin
		if(nreset = '0')            then CS <= init;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg

	sm_combinatorial: 
	process 
	(CS, ltch_save, ltch_restore, done, FLSH_Bus_Permit, delay, bus_permit)
	begin
		-- default signal values
		Bootup 			 <= '0';
		SaveAll_out		 <= '0';
		RestoreAll_out	 <= '0';
		FLSH_Bus_Request <= '1';
		ena_delay		 <= '0';
                            
		-------------------------
		case CS is
			when init	=>							
				FLSH_Bus_Request <= '0'; -- Otherwise = '1'
				Bootup <= '1'; -- this will set ltch_restore = '1'
				
				ena_delay <= '1';
				if   (delay    = X"FF00") then 		NS <= make_request;
				else								NS <= init;
				end if;
				
			when idle   =>
				FLSH_Bus_Request <= '0'; -- Otherwise = '1'
				if   (ltch_save    = '1') then 		NS <= make_request;
				elsif(ltch_restore = '1') then 		NS <= make_request;
				else								NS <= idle;
				end if;
			    
			when make_request  =>										
				if  (bus_permit = '1') then 		
					if   (ltch_save    = '1') then 	NS <= save_active;
					elsif(ltch_restore = '1') then 	NS <= restore_active;
					else							NS <= idle;
					end if;
				else								NS <= make_request;
				end if;
			    
			when save_active  =>						
				SaveAll_out <= ltch_save;
				
				if   (done    = '1') then 			NS <= transf_done;
				else								NS <= save_active;
				end if;

			when restore_active  =>						
				RestoreAll_out <= ltch_restore;
				
				if   (done    = '1') then 			NS <= transf_done;
				else								NS <= restore_active;
				end if;

			when transf_done =>						NS <= idle;

			when others  =>						NS <= idle;
		end case; 
				
	end process;

end;