Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

USE work.pkg_081006.all; 

-------------------------------------------------------------------- 
-- Description:

entity LTC2641_Interface is
	port(
		 clk           : in std_logic; -- global Logic Clock
		 reset_n       : in std_logic; -- global Logic Reset
		 dac_data      : in std_logic_vector(15 downto 0); 
		 data_strb     : in std_logic;   
		 
		 dac_ncs 		: out std_logic; 
		 dac_sclk 		: out std_logic;
		 dac_din	   	: out std_logic;
		 dac_ready		: out std_logic
		);
end LTC2641_Interface;
	
architecture main of LTC2641_Interface is
	-- Shift Register declarations -----------------------------------
	signal sr         	: std_logic_vector(15 downto 0); --
	signal shift       	: std_logic;
	signal load			: std_logic;
	signal dac_cs		: std_logic;
	
	-- Process Trigger LED -------------------------------------------
	signal strb_led		: std_logic;
	signal strb			: std_logic;

	-- Process Trigger Latch -----------------------------------------
	signal strb_ltch   	: std_logic;
	signal clr_strb		: std_logic;
	signal go	      	: std_logic;
	
	-- Counter/Timer declarations ------------------------------------
	signal nclr_cnt    : std_logic;
	signal srcount     : std_logic_vector (4 downto 0);
	
	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(init, idle, pause, xmit_data, done);
	signal CS, NS  : STATE_TYPE;
	
--===begin===begin===begin===begin===begin===begin===begin===begin===
begin -- main --------------------------------------------------------

-- Process Trigger Latch ---------------------------------------------
STROBE_LED: process(clk, reset_n, data_strb)
	begin
		if   (reset_n = '0') 	then strb_led <= '0';
		elsif rising_edge(clk) 	then strb_led <= data_strb;
		end if;
	end process;

	strb <= data_strb AND NOT strb_led;
	
-- Process Trigger Latch ---------------------------------------------
STROBE_LATCH: process(clk, reset_n, strb)
	begin
		if   (reset_n = '0') 		then go <= '0';
		elsif rising_edge(clk) then
			if   (strb = '1')		then go <= '1';
			elsif(clr_strb = '1')	then go <= '0';
			end if;
		end if;
	end process;

-- Shift Registers ---------------------------------------------------
    shift_reg: process (clk, reset_n, shift, load)
	begin
		if(reset_n = '0') 		then sr <= (others => '0');
		elsif rising_edge(clk) then
			if (shift = '1') then 
			  for i in 0 to 14 	loop sr(i+1) <= sr(i); end loop;
			elsif(load = '1') 	then sr <= dac_data;
			end if;
		end if;
    end process;
    
	dac_din 	<= sr(15);
    dac_sclk	<= (NOT clk) AND (shift);
	dac_ncs		<= NOT dac_cs;
	
-- Shift Counter -------------------------------------------------------
SHIFT_COUNT: process (clk, shift) -- bit counter for shift register
	begin
		if(shift = '0')     then srcount <= (others => '0');
		elsif rising_edge(clk) then srcount <= srcount + 1;
		end if;
	end process;
				 
------------------------------------------------------------------------	
-- State Machine -------------------------------------------------------
	sm_reg: process (clk, reset_n)	
	begin
		if(reset_n = '0')        then CS <= init;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm: process (CS, go, srcount)
	begin
		-- default signal values
		clr_strb  	<= '0';
		shift  		<= '0';
		load		<= '0';
		dac_cs		<= '0';
		dac_ready	<= '0';
		-------------------------
		case CS is
			when init   	=>				NS <= idle;
				
			when idle   	=>
				dac_ready	<= '1';
				load <= '1';
				if(go = '1')		then	NS <= pause;
				else    					NS <= idle;
				end if;	
						    
			when pause	 	=>				NS <= xmit_data;
				clr_strb	<= '1';
				dac_cs		<= '1';

			when xmit_data 	=>
				clr_strb	<= '1';
				shift 		<= '1';
				dac_cs		<= '1';
				
				if(srcount = X"f")	then	NS <= done;
				else						NS <= xmit_data;
				end if;
				
			when done 		=>				NS <= idle;
				dac_cs		<= '1';
				dac_ready	<= '1';
				
			when others 	=>				NS <= init;
		end case; 				
	end process; -- sm
	
end main;

