Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

entity MI_Phaselock_Timing_v3 is
	port(clk        	: in std_logic;
		 nreset     	: in std_logic;
		 ARM_MIPL		: in std_logic;
		 Trig_MIPL		: in std_logic;
		 Slope_Up_nDn	: in std_logic;
		 Beam_Gate		: in std_logic;
		 
		 Gate_MIPL_Always 	: in std_logic;
		 Ena_MIPL_Delay		: in std_logic;
		 
		 MIPL_Armed		: out std_logic;
		 Run_MIPL		: out std_logic;
		 End_MIPL		: out std_logic
		);
end MI_Phaselock_Timing_v3;
	
architecture main of MI_Phaselock_Timing_v3 is

signal arm			: std_logic;
signal trig			: std_logic;
signal dn_trig		: std_logic;

signal timer		: std_logic_vector(23 downto 0);
signal timer_ena	: std_logic;

signal use_mipl_delay	: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, armed, test_use_delay, delay_mipl, triggered, clr_timer2, go, done);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 

sync_ARM_Pulse: led
port map(sig_in => ARM_MIPL, clk => clk, clrn => nreset, pls_out => arm);
	
sync_TRIG: led
port map(sig_in => Trig_MIPL AND (Beam_Gate OR Gate_MIPL_Always), 
		 clk => clk, clrn => nreset, pls_out => trig);
	
sync_DN: led
port map(sig_in => not Slope_Up_nDn, clk => clk, 
			clrn => nreset, pls_out => dn_trig);

sync_ENA_MIPL_DELAY: process(clk, nreset, Ena_MIPL_Delay)
begin
	if  (nreset = '0') then 	use_mipl_delay <= '0';
	elsif rising_edge(clk) then
		if  (Ena_MIPL_Delay = '1') then use_mipl_delay <= '1';
		else							use_mipl_delay <= '0';
		end if;
	end if;
	
end process;

PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 	timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '1') then 	timer <= timer + 1;
		else						timer <= (others => '0');
		end if;
	end if;
end process;

----------------------------------------------------------
----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, arm, trig, dn_trig, timer, use_mipl_delay)
begin
	-- default outputs --
	timer_ena 	<= '0'; 
	MIPL_Armed	<= '0';
	Run_MIPL	<= '0';
	End_MIPL	<= '0';
		
	case CS is
		when idle   =>
			if (arm = '1') 	then	NS <= armed;
			else					NS <= idle;
			end if;
	
		when armed  =>
			MIPL_Armed	<= '1';
			timer_ena	<= '1';
			if   (timer = X"30D400") then	NS <= idle; -- time out at 40 ms
			elsif(trig = '1') then 			NS <= test_use_delay; --triggered;
			else							NS <= armed;
			end if;
		
		when test_use_delay =>					
			timer_ena	<= '0'; --opportunity to clear timer
			
			if  (use_mipl_delay = '1') then	NS <= delay_mipl;
			else							NS <= triggered;
			end if;
		
		when delay_mipl =>			
			timer_ena	<= '1';
			if   (timer = X"009C40") then	NS <= triggered; -- delay 500us
			else							NS <= delay_mipl;
			end if;
			
		when triggered =>			
			timer_ena	<= '1';
			if   (timer = X"0C3500") then	NS <= idle; -- No Beam or other error
			elsif(dn_trig = '1')     then 	NS <= clr_timer2; --go;
			else							NS <= triggered;
			end if;
			
		when clr_timer2 =>					NS <= go;
			timer_ena	<= '0';

		when go =>
			Run_MIPL	<= '1';
			timer_ena	<= '1';
			if   (timer = X"0C3500") then	NS <= done; -- wait 10 ms
			else							NS <= go;
			end if;
			
		when done  =>						NS <= idle;
			End_MIPL	<= '1';
			
		when others =>						NS <= idle;
	end case; 
			
end process;

end main;