--------------------------------------------------------------------
-- Description: This logic is intended to average a 16 bit value
-- and then write out the average when completed.  It is NOT a 
-- running or boxcar average.  The data rate out is effectively
-- reduced.

Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;
USE work.pkg_081006.all;  -- my definitions and entities

-------------------------------------------------------------------- 

entity avg_bin_offset is
	port(clk, nreset 	: in std_logic;
		 data 			: in std_logic_vector(13 downto 0);
		 strb 			: in std_logic;
		 avg_out  		: out std_logic_vector(13 downto 0);
		 strb_out 		: out std_logic;
		 sum    		: buffer std_logic_vector(23 downto 0);
		 slope 			: out std_logic;
		 polarity		: out std_logic
		);
end avg_bin_offset;
	
architecture main of avg_bin_offset is
	
	CONSTANT AVERAGED_NUM_OF_POINTS: std_logic_vector(7 downto 0) := X"10";-- Avg 16 samples
	
	signal data_in: std_logic_vector(13 downto 0); -- Data in register output
	signal inp_ena: std_logic;
	
--	signal sum    : std_logic_vector(23 downto 0); -- Sum register input
	signal result : std_logic_vector(23 downto 0); -- Sum register output
	signal sum_ena: std_logic;
	signal clr_sum, clr_sumn: std_logic;
	
	signal avg, avg2 : std_logic_vector(13 downto 0);
	
	signal out_ena: std_logic;
	
	signal done, go : std_logic;
	
	-- Sum counter declarations --------------------------------------
	signal ctr_ena: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, getsmp, do_sum, s_wait, getsum, chkcnt, getavg, wr_avg);
	signal CS, NS : STATE_TYPE;

begin -- main --------------------------------------------------------
	INPREG: dffr  -- Input register to latch the next sample
	generic map(WID14)  
	port map(d=>data, clk=>clk, clrn=>nreset, 
			 ena=>inp_ena, prn=>'1', q=>data_in
			); 

	SUMREG: dffr   -- A register that accepts the adders output
	generic map(WID24)  
	port map(d=>sum, clk=>clk, clrn=>clr_sumn, 
			 ena=>sum_ena, prn=>'1', q=>result
			); 
			
	sum <= result + data_in;
	clr_sumn <= not ((not nreset) or clr_sum);
			
	SUMCNT: counter  -- Counter that counts the number of samples per average
	generic map(WID8)
	port map ( clk=>clk, clrn=>clr_sumn, ena=>ctr_ena, load=>'0', updn=>'1',
			   ld_data  => (others => '0'),
			   flag_cnt => AVERAGED_NUM_OF_POINTS,
			   flag_out => done,
			   cnt_out  => open
			 );

	OUTPREG: dffr  -- Output register to latch the average
	generic map(WID14)  
	port map(d=>result(13+4 downto 4), clk=>clk, clrn=>nreset, 
			 ena=>out_ena, prn=>'1', q=>avg
			);
			 
	SLOPREG: dffr  -- Output register to latch the "Previous" average
	generic map(WID14)  
	port map(d=>avg, clk=>clk, clrn=>nreset, 
			 ena=>out_ena, prn=>'1', q=>avg2
			); 
						
	GOLED: led  -- Syncronizing leading edge detector
	port map(sig_in => strb, clk => clk, clrn => nreset, pls_out => go );
	
-- State Machine definition ---------------------------
	sm_reg: process (clk, nreset)	
	begin
		if(nreset='0')              then CS <= idle;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: process (CS, go, done)
	begin
		case CS is
			when idle   =>
				if(go='1')then NS <= getsmp;
				else           NS <= idle  ;
				end if;
				-- outputs --
				inp_ena <= '0';
				sum_ena <= '0'; 
				ctr_ena <= '0';
				strb_out <= '0';
				clr_sum <= '0';
				out_ena <= '0';
				--------------------
			when getsmp =>
				NS <= do_sum;
				-- outputs --
				inp_ena <= '1';
				sum_ena <= '0'; 
				ctr_ena <= '0';				
				strb_out <= '0';
				clr_sum <= '0';
				out_ena <= '0';
				--------------------
			when do_sum =>
				NS <= s_wait;
				-- outputs --
				inp_ena <= '0';
				sum_ena <= '0'; 
				ctr_ena <= '1';				
				strb_out <= '0';
				clr_sum <= '0';
				out_ena <= '0';
				--------------------
			when s_wait =>
				NS <= getsum; -- add a counter if a longer pause is needed
				-- outputs --
				inp_ena <= '0';
				sum_ena <= '0'; 
				ctr_ena <= '0';				
				strb_out <= '0';
				clr_sum <= '0';
				out_ena <= '0';
				----------------
			when getsum =>
				NS <= chkcnt;
				-- outputs --
				inp_ena <= '0';
				sum_ena <= '1'; 
				ctr_ena <= '0';			
				strb_out <= '0';
				clr_sum <= '0';
				out_ena <= '0';
				----------------
			when chkcnt =>
				if(done='1')then NS <= getavg;
				else             NS <= idle  ;
				end if;
				-- outputs --
				inp_ena <= '0';
				sum_ena <= '0'; 
				ctr_ena <= '0';			
				strb_out <= '0';
				clr_sum <= '0';
				out_ena <= '0';
				----------------
			when getavg =>
				NS <= wr_avg;
				-- outputs --
				inp_ena <= '0';
				sum_ena <= '0'; 
				ctr_ena <= '0';			
				strb_out <= '0';
				clr_sum <= '0';
				out_ena <= '1';
				----------------
			when wr_avg =>
				NS <= idle;
				-- outputs --
				inp_ena <= '0';
				sum_ena <= '0'; 
				ctr_ena <= '0';			
				strb_out <= '1';
				clr_sum <= '1';
				out_ena <= '0';
				----------------
		end case; 
				
	end process; -- sm_combinatorial

-- Syncronized Outputs ----------------------------
	process(clk, avg, avg2)
	begin
		if rising_edge(clk) then
			avg_out <= avg;
			if (avg > avg2) then slope <= '1';
			else                 slope <= '0';
			end if;
		end if;
	end process;
	
	process(clk, avg)
	begin
		if rising_edge(clk) then
--			if (avg > X"2000") then polarity <= '1';
			if (avg > X"1800") then polarity <= '1';
			else                 	polarity <= '0';
			end if;
		end if;
	end process;

end main;

