--------------------------------------------------------------------
-- Description: This logic is intended to average a 16 bit value
-- and then write out the average when completed.  It is NOT a 
-- running or boxcar average.  The data rate out is effectively
-- reduced.

Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;
USE work.pkg_081006.all;  -- my definitions and entities

-------------------------------------------------------------------- 

entity avg_bin_offset_v2 is
	port(clk, nreset 	: in std_logic;
		 data 			: in std_logic_vector(13 downto 0);
		 strb 			: in std_logic;
		 ZC_Thresh		: in std_logic_vector(13 downto 0);
		 
		 avg_out  		: out std_logic_vector(13 downto 0);
		 strb_out 		: out std_logic;
		 sum    		: buffer std_logic_vector(23 downto 0);
		 Slope_Out		: out std_logic;
		 polarity		: out std_logic;
		 Min_Trig		: out std_logic;
		 Max_Trig		: out std_logic
		);
end avg_bin_offset_v2;
	
architecture main of avg_bin_offset_v2 is
	
	CONSTANT AVERAGED_NUM_OF_POINTS: std_logic_vector(7 downto 0) := X"10";-- Avg 16 samples
	
	signal data_in: std_logic_vector(13 downto 0); -- Data in register output
	signal inp_ena: std_logic;
	
--	signal sum    : std_logic_vector(23 downto 0); -- Sum register input
	signal result : std_logic_vector(23 downto 0); -- Sum register output
	signal sum_ena: std_logic;
	signal clr_sum, clr_sumn: std_logic;
	
	signal avg, avg2 : std_logic_vector(13 downto 0);
	signal min, max	 : std_logic_vector(13 downto 0);
	signal slope	 : std_logic;
	
	signal get_min, get_max : std_logic;
	
	signal out_ena: std_logic;
	
	signal done, go : std_logic;
	
	-- Sum counter declarations --------------------------------------
	signal ctr_ena: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, getsmp, do_sum, s_wait, getsum, chkcnt, getavg, wr_avg);
	signal CS, NS : STATE_TYPE;

begin -- main --------------------------------------------------------
	INPREG: dffr  -- Input register to latch the next sample
	generic map(WID14)  
	port map(d=>data, clk=>clk, clrn=>nreset, 
			 ena=>inp_ena, prn=>'1', q=>data_in
			); 

	SUMREG: dffr   -- A register that accepts the adders output
	generic map(WID24)  
	port map(d=>sum, clk=>clk, clrn=>clr_sumn, 
			 ena=>sum_ena, prn=>'1', q=>result
			); 
			
	sum <= result + data_in;
	clr_sumn <= not ((not nreset) or clr_sum);
			
	SUMCNT: counter  -- Counter that counts the number of samples per average
	generic map(WID8)
	port map ( clk=>clk, clrn=>clr_sumn, ena=>ctr_ena, load=>'0', updn=>'1',
			   ld_data  => (others => '0'),
			   flag_cnt => AVERAGED_NUM_OF_POINTS,
			   flag_out => done,
			   cnt_out  => open
			 );

	OUTPREG: dffr  -- Output register to latch the average
	generic map(WID14)  
	port map(d=>result(13+4 downto 4), clk=>clk, clrn=>nreset, 
			 ena=>out_ena, prn=>'1', q=>avg
			);
			 
	SLOPREG: dffr  -- Output register to latch the "Previous" average
	generic map(WID14)  
	port map(d=>avg, clk=>clk, clrn=>nreset, 
			 ena=>out_ena, prn=>'1', q=>avg2
			); 
						
	GOLED: led  -- Syncronizing leading edge detector
	port map(sig_in => strb, clk => clk, clrn => nreset, pls_out => go );
	
-- State Machine definition ---------------------------
	sm_reg: process (clk, nreset)	
	begin
		if(nreset='0')              then CS <= idle;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: process (CS, go, done)
	begin
		-- default outputs --
		inp_ena 	<= '0';
		sum_ena 	<= '0'; 
		ctr_ena 	<= '0';
		strb_out	<= '0';
		clr_sum 	<= '0';
		out_ena 	<= '0';
		
		case CS is
			when idle   =>
				if(go='1')then 		NS <= getsmp;
				else           		NS <= idle  ;
				end if;
				
			when getsmp =>			NS <= do_sum;
				inp_ena <= '1'; -- latch the current sample
				
			when do_sum =>			NS <= s_wait;
				ctr_ena <= '1';	-- increment the count of samples in the average.			
				
			when s_wait =>			NS <= getsum; 
								-- add a counter if a longer pause is needed
				
			when getsum =>			NS <= chkcnt;
				sum_ena <= '1'; -- latch sum into the accumulator register
				
			when chkcnt =>
				if(done='1')then 	NS <= getavg;
				else             	NS <= idle  ;
				end if;
				
			when getavg =>			NS <= wr_avg;
				out_ena <= '1'; 
				
			when wr_avg =>			NS <= idle;
				strb_out <= '1';
				clr_sum <= '1';
				----------------
		end case; 
				
	end process; -- sm_combinatorial

-- Syncronized Outputs ----------------------------
	process(clk, avg, avg2)
	begin
		if rising_edge(clk) then
			avg_out <= avg;
			if (avg > avg2) then slope <= '1';
			else                 slope <= '0';
			end if;
		end if;
	end process;

	Slope_Out <= slope;

	GETMIN_LED: led  -- Syncronizing leading edge detector
	port map(sig_in => slope, clk => clk, clrn => nreset, pls_out => get_min );

	GETMAX_LED: led  -- Syncronizing leading edge detector
	port map(sig_in => not slope, clk => clk, clrn => nreset, pls_out => get_max );

	process(clk, avg, get_min, get_max, ZC_Thresh)
	begin
		if rising_edge(clk) then
			if   (get_min = '1') then min <= avg + ZC_Thresh;
			elsif(get_max = '1') then max <= avg - ZC_Thresh;
			end if;
		end if;
	end process;

	process(clk, avg, min, slope)
	begin
		if rising_edge(clk) then
			if   ((avg > min) AND (slope = '1')) then Min_Trig <= '1';
			else									  Min_Trig <= '0';
			end if;
		end if;
	end process;
		
	process(clk, avg, max, slope)
	begin
		if rising_edge(clk) then
			if   ((avg < max) AND (slope = '0')) then Max_Trig <= '1';
			else									  Max_Trig <= '0';
			end if;
		end if;
	end process;
	
	process(clk, avg)
	begin
		if rising_edge(clk) then
--			if (avg > X"2000") then polarity <= '1';
--			if (avg > X"1800") then polarity <= '1';
			if (avg > ZC_Thresh) then polarity <= '1';
			else                 	  polarity <= '0';
			end if;
		end if;
	end process;

end main;

